#include "common.h"

/*ADC时钟*/
enum ADC_CLOCK
{
    ADC_CLOCK_FSYS_DIV_2 = 0,//Fsys/2
    ADC_CLOCK_FSYS_DIV_4,//Fsys/4
    ADC_CLOCK_FSYS_DIV_8,//Fsys/8
    ADC_CLOCK_FSYS_DIV_16,//Fsys/16
    ADC_CLOCK_FSYS_DIV_32,//Fsys/32
    ADC_CLOCK_FSYS_DIV_64,//Fsys/64
    ADC_CLOCK_FSYS_DIV_128,//Fsys/128
    ADC_CLOCK_FSYS_DIV_256,//Fsys/256
};

/*ADC参考电压*/
enum ADC_VERF
{
    ADC_VERF_IN_1_2V = 0,//内部1.2V
    ADC_VERF_IN_2_0V,//内部2.0V
    ADC_VERF_IN_2_4V,//内部2.4V
    ADC_VERF_IN_3_0V,//内部3.0V
    ADC_VERF_VDD,//VDD
};

/*ADC选择通道*/
enum ADC_CHANNEL//通道0~22
{
    ADC_CHANNEL_0 = 0,
    ADC_CHANNEL_1,
    ADC_CHANNEL_2,
    ADC_CHANNEL_3,
    ADC_CHANNEL_4,
    ADC_CHANNEL_5,
    ADC_CHANNEL_6,
    ADC_CHANNEL_7,
    ADC_CHANNEL_8,
    ADC_CHANNEL_9,
    ADC_CHANNEL_10,
    ADC_CHANNEL_11,
    ADC_CHANNEL_12,
    ADC_CHANNEL_13,
    ADC_CHANNEL_14,
    ADC_CHANNEL_15,
    ADC_CHANNEL_16,
    ADC_CHANNEL_17,
    ADC_CHANNEL_18,
    ADC_CHANNEL_19,
    ADC_CHANNEL_20,
    ADC_CHANNEL_21,
    ADC_CHANNEL_22,
    
    ADC_CHANNEL31_ACMP_VREF,
    ADC_CHANNEL31_OP1_O,
    ADC_CHANNEL31_OP0_O,
    ADC_CHANNEL31_BRG_1_2V,
    ADC_CHANNEL31_GND,
    ADC_CHANNEL31_VDD,
};

/*ADC设置时钟（不超过1MHz）*/
void ADCSetClk(enum ADC_CLOCK adcClk)
{
    ADCON1 &= ~(BIT6 | BIT5 | BIT4);
    ADCON1 |= (adcClk << 4);
}

/*ADC设置参考电压*/
void ADCSetVREF(enum ADC_VERF adcVref)
{
    ADCLDO = 0x00;
    if (adcVref <= ADC_VERF_IN_3_0V)
    {
        ADCLDO = BIT7 | (adcVref << 5);
    }
}

/*ADC转换*/
uint16_t ADCCovert(enum ADC_CHANNEL adcChannel)
{
    uint16_t adcRes = 0x0000;
    
    /*设置通道*/
    ADCON0 &= ~BIT7;
    ADCON1 &= ~(BIT3 | BIT2 | BIT1 | BIT0);
    if (adcChannel <= ADC_CHANNEL_15)
    {
        ADCON1 |= adcChannel;
    }
    else if (adcChannel <= ADC_CHANNEL_22)
    {
        ADCON0 |= BIT7;
        ADCON1 |= (adcChannel - ADC_CHANNEL_16);
    }
    else if ((adcChannel>=ADC_CHANNEL31_ACMP_VREF) && (adcChannel<=ADC_CHANNEL31_VDD))
    {
        ADCON0 |= BIT7;
        ADCON1 |= (BIT3 | BIT2 | BIT1 | BIT0);
        
        ADCON0 &= ~(BIT4 | BIT3 | BIT2);
        ADCON0 |= ((adcChannel - ADC_CHANNEL31_ACMP_VREF + 1) << 2);
    }
    
    _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
    _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
    
    /*转换*/
    ADCON0 |= BIT1;
    while (ADCON0 & BIT1);
    
    if (ADCON0 & BIT6)
    {
        adcRes = ADRESH & 0x0F;
        adcRes <<= 8;
        adcRes += ADRESL;
    }
    else
    {
        adcRes = ADRESH;
        adcRes <<= 4;
        adcRes += ((ADRESL >> 4) & 0x0F);
    }
    
    return adcRes;
}

/*获取VDD电压（前提是参考电压采用VDD）*/
float GetVDDVoltage(void)
{
    float vddVol = 0.0;
    uint16_t adcRes = 0x0000;
    
    adcRes = ADCCovert(ADC_CHANNEL31_BRG_1_2V);
    vddVol = 1.2 * 4096.0 / adcRes;
    
    return vddVol;
}

/*main*/
void main(void)
{
    float vddVol = 0x0000;
    uint16_t channelADCRes = 0x0000;
    
    uint16_t i = 0x0000;
    
    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();
    
    /*ADC管脚设置为模拟输入（本例用P17做通道3）*/
    P17CFG = 0x01;
    P1TRIS &= ~BIT7;
    
    /*使能ADC*/
    ADCON1 |= BIT7;
    
    /*ADC设置时钟（不超过1MHz）*/
    ADCSetClk(ADC_CLOCK_FSYS_DIV_64);
    
    /*ADC设置参考电压*/
    ADCSetVREF(ADC_VERF_VDD);
    
    while (1)
    {
        /*获取VDD电压（前提是参考电压采用VDD）*/
        vddVol = GetVDDVoltage();
        printf("VDD = %.2fV\r\n", vddVol);
        
        /*ADC转换*/
        channelADCRes = ADCCovert(ADC_CHANNEL_3);
        printf("Channel result = %d\r\n\r\n", (int)channelADCRes);
        
         for (i=0; i<60000; i++);
			for (i=0; i<60000; i++);
//			for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    }
}