#include "common.h"

/*Flash锁定、解锁*/
void FlashLock(void)
{
    MADRL = 0xFF;
    MADRH = 0xFF;
    
    MCTRL = 0x10;
    
    MLOCK = 0x00;
}
void FlashUnlock(void)
{
    MLOCK = 0xAA;
}

/*Flash读、扇区擦除、写（area=0：Flash区；area=1：DataFlash区）*/
void FlashRead(uint8_t area, uint16_t addr, uint8_t* pDat, uint16_t datCount)
{
    uint8_t mctrl = 0x00;
    
    /*解锁*/
    FlashUnlock();
    
    /*读*/
    if (0 == area)
    {
        mctrl = BIT0;
    }
    else
    {
        mctrl = BIT4 | BIT0;
    }
    
    while (datCount --)
    {
        MADRH = (addr >> 8) & 0x00FF;
        MADRL = addr & 0x00FF;
        
        MCTRL = mctrl;
        _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
        while (MCTRL & BIT0);
        
        *pDat = MDATA;
        
        addr ++;
        pDat ++;
    }
    
    /*锁定*/
    FlashLock();
}
void FlashEraseSector(uint8_t area, uint16_t addr)
{
    uint16_t startAddr = addr / 512 * 512;
    
    /*解锁*/
    FlashUnlock();
    
    /*擦除*/
    MADRH = (startAddr >> 8) & 0x00FF;
    MADRL = startAddr & 0x00FF;
    
    MDATA = 0xFF;
    
    if (0 == area)
    {
        MCTRL = BIT3 | BIT2 | BIT0;//“操作”跟“启动”必须同时操作
    }
    else
    {
        MCTRL = BIT4 | BIT3 | BIT2 | BIT0;
    }
    _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
    while (MCTRL & BIT0);
    
    /*锁定*/
    FlashLock();
}
uint8_t FlashWrite(uint8_t area, uint16_t addr, uint8_t* pDat, uint16_t datCount)
{
    uint8_t mctrl = 0x00;
    
    /*解锁*/
    FlashUnlock();
    
    /*写*/
    if (0 == area)
    {
        mctrl = BIT3 | BIT0;
    }
    else
    {
        mctrl = BIT4 | BIT3 | BIT0;
    }
    
    while (datCount --)
    {
        MADRH = (addr >> 8) & 0x00FF;
        MADRL = addr & 0x00FF;
        
        MDATA = *pDat;
        
        MCTRL = mctrl;
        _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
        while (MCTRL & BIT0);
        
        addr ++;
        pDat ++;
        
        if (MCTRL & BIT5)
        {
            MLOCK = 0x00;
            return 0;
        }
    }
    
    /*锁定*/
    FlashLock();
    
    return 1;
}

/*main*/
void main(void)
{
    uint8_t datW[30] = {0x00};
    uint8_t datR[30] = {0x00};

    uint16_t i = 0x0000;

    /*Flash锁定*/
    FlashLock();
    
    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();
    
    /*操作Flash*/
    FlashEraseSector(0, 0x2800);//扇区擦除
    for (i=0; i<30; i++)//写
    {
        datW[i] = i;
    }
    if (0 == FlashWrite(0,0x2800,datW,30))
    {
        printf("Write flash fail");
        while (1);
    }
    for (i=0; i<30; i++)//读
    {
        datR[i] = 0x00;
    }
    FlashRead(0, 0x2800, datR, 30);
    printf("\r\n");
    for (i=0; i<30; i++)
    {
        printf("%02d ", (int)datR[i]);
    }
    printf("\r\n");
    

    /*操作DataFlash*/
    FlashEraseSector(1, 0x0003);//扇区擦除
    for (i=0; i<30; i++)//写
    {
        datW[i] = i + 3;
    }
    if (0 == FlashWrite(1,0x0003,datW,30))
    {
        printf("Write data flash fail");
        while (1);
    }
    for (i=0; i<30; i++)//读
    {
        datR[i] = 0x00;
    }
    FlashRead(1, 0x0003, datR, 30);
    printf("\r\n");
    for (i=0; i<30; i++)
    {
        printf("%02d ", (int)datR[i]);
    }
    printf("\r\n");

    while (1);
}