#include "common.h"

/*I2C从机设备地址、数据地址*/
#define I2C_SLAVE_DEVICE_ADDRESS 0xA0
#define I2C_SLAVE_DATA_ADDRESS 0x00

/*I2C写（流程参照写AT24C02，且不考虑“换页”的情况）*/
uint8_t I2CWrite(uint8_t slaDevAddr, uint8_t slaDatAddr, uint8_t* pDat, uint16_t datCount)
{
    uint8_t flagOK = 1;
    
    /*STA，devAddr+W，datAddr*/
    if (1 == flagOK)
    {
        I2CMSA = slaDevAddr & 0xFE;
        I2CMBUF = slaDatAddr;
        I2CMCR = 0x03;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
    }

    /*dat[0]~dat[n-1]+receive ACK*/
    while ((1==flagOK) && (datCount>1))
    {
        I2CMBUF = *pDat;
        I2CMCR = 0x01;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
        
        pDat ++;
        datCount --;
    }

    /*dat[n]+receive ACK，STO*/
    if (1 == flagOK)
    {
        I2CMBUF = *pDat;
        I2CMCR = 0x05;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
    }
    
    /*STO if error*/
    if (0 == flagOK)
    {
        I2CMCR = 0x04;
    }
    
    return flagOK;
}

/*I2C读（流程参照读AT24C02，且不考虑“换页”的情况）*/
uint8_t I2CRead(uint8_t slaDevAddr, uint8_t slaDatAddr, uint8_t* pDat, uint16_t datCount)
{
    uint8_t flagOK = 1;
    
    /*STA，devAddr+W，datAddr*/
    if (1 == flagOK)
    {
        I2CMSA = slaDevAddr & 0xFE;
        I2CMBUF = slaDatAddr;
        I2CMCR = 0x03;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
    }

    /*STA，devAddr+R，dat[0]+send ACK*/
    if (1 == flagOK)
    {
        I2CMSA = slaDevAddr | 0x01;
        I2CMCR = 0x0B;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
        
        *pDat = I2CMBUF;
        
        pDat ++;
        datCount --;
    }
    
    /*dat[1]~dat[n-1]+send ACK*/
    while ((1==flagOK) && (datCount>1))
    {
        I2CMCR = 0x09;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
        
        *pDat = I2CMBUF;
        
        pDat ++;
        datCount --;
    }

    /*dat[n]+send NACK、STO*/
    if (1 == flagOK)
    {
        I2CMCR = 0x05;
        while ((EIF2&0x40) == 0x00);
        
        if ((I2CMCR&0x1E) != 0x00)
        {
            flagOK = 0;
        }
        I2CMCR = 0x00;
        
        *pDat = I2CMBUF;
    }
    
    /*STO if error*/
    if (0 == flagOK)
    {
        I2CMCR = 0x04;
    }
    
    return flagOK;
}

/*I2C初始化（P04做SCL，P05做SDA）*/
void I2CInit(void)
{
    /*P04、P05分别做I2C的SCL、SDA管脚*/
    P04CFG = 0x04;
    PS_SCL = 0x04;
    P05CFG = 0x04;
    PS_SDA = 0x05;
    
    /*P04、P05开漏输出*/
    P0TRIS |= (BIT5 | BIT4);
    P0OD |= (BIT5 | BIT4);
    P0 |= (BIT5 | BIT4);
    
//     /*P04、P05打开内部上拉电阻（注意：内部上拉电阻是32K；当I2C时钟较快时，建议采用4.7K左右外部上拉电阻）*/
//     P0UP |= (BIT5 | BIT4);
    
    /*时钟*/
//     I2CMTP = 5;//400Kb/s
    I2CMTP = 23;//100Kb/s
    
    /*清标志*/
    I2CMCR = 0x00;
}

/*main*/
void main(void)
{
    uint8_t datR[20] = {0x00};
    uint8_t datW[20] = {0x00};

    uint16_t i = 0x0000;

    /*P07推挽输出（仅用于测试）*/
    P0TRIS |= BIT7;
    P0OD &= ~BIT7;
    P0_7 = 1;

    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();

    /*I2C初始化（P04做SCL，P05做SDA）*/
    I2CInit();

    /*延时一段时间，等待从机准备好*/
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);

    printf("Master is going to read and write slave\r\n");
   
    /*I2C写、读*/
    for (i=0; i<20; i++)//写
    {
        datW[i] = i;
    }
    I2CWrite(I2C_SLAVE_DEVICE_ADDRESS, I2C_SLAVE_DATA_ADDRESS, datW, 20);
    
    for (i=0; i<20; i++)//读（配合本例I2C从机，读回数据正常是0x00,0x01,0x02......）
    {
        datR[i] = 0x00;
    }
    I2CRead(I2C_SLAVE_DEVICE_ADDRESS, I2C_SLAVE_DATA_ADDRESS, datR, 20);
    for (i=0; i<20; i++)
    {
        printf("%02X ", (int)datR[i]);
        datR[i] = 0x00;
    }
    printf("\r\n");
    
    /*延时一段时间*/
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);

    /*I2C再次写、读*/
    for (i=0; i<20; i++)//写
    {
        datW[i] = i;
    }
    I2CWrite(I2C_SLAVE_DEVICE_ADDRESS, I2C_SLAVE_DATA_ADDRESS, datW, 20);
    
    for (i=0; i<20; i++)//读（配合本例I2C从机，读回数据正常是0x00,0x01,0x02......）
    {
        datR[i] = 0x00;
    }
    I2CRead(I2C_SLAVE_DEVICE_ADDRESS, I2C_SLAVE_DATA_ADDRESS, datR, 20);
    for (i=0; i<20; i++)
    {
        printf("%02X ", (int)datR[i]);
        datR[i] = 0x00;
    }
    printf("\r\n");
    
    printf("Master has stop to read and write slave\r\n\r\n");
    
    /*主机P07和从机P07接在一起，此处拉低P07一段时间，以通知从机printf接收到的数据（仅用于测试）*/
    P0_7 = 0;
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    P0_7 = 1;
    
    while (1);
}