#include "common.h"

/*I2C从机设备地址、数据地址*/
#define I2C_SLAVE_DEVICE_ADDRESS 0xA0
#define I2C_SLAVE_DATA_ADDRESS 0x00

/*数据*/
xdata uint8_t datR[80] = {0x00};
uint8_t datRIndex = 0x00;
xdata uint8_t datW[20] = {0x00};
uint8_t datWIndex = 0x00;

/*I2C初始化（P04做SCL，P05做SDA）*/
void I2CInit(void)
{
    /*P04、P05分别做I2C的SCL、SDA管脚*/
    P04CFG = 0x04;
    PS_SCL = 0x04;
    P05CFG = 0x04;
    PS_SDA = 0x05;
    
    /*P04、P05开漏输出*/
    P0TRIS |= (BIT5 | BIT4);
    P0OD |= (BIT5 | BIT4);
    P0 |= (BIT5 | BIT4);
    
//     /*P04、P05打开内部上拉电阻（注：本例由I2C主机端提供上拉电阻）*/
//     P0UP |= (BIT5 | BIT4);
    
    /*做从机*/
    I2CSCR |= 0x01;
    
    /*从机地址*/
    I2CSADR = I2C_SLAVE_DEVICE_ADDRESS >> 1;
    
    /*使能I2C中断*/
    EIE2 |= 0x40;
}

/*main*/
void main(void)
{
    uint16_t i = 0x0000;
    
    /*P07内部上拉做输入（仅用于测试）*/
    P0TRIS &= ~BIT7;
    P0UP |= BIT7;
    P0_7 = 1;

    /*准备数据（仅用于测试）*/
    for (i=0; i<80; i++)
    {
        datR[i] = 0x00;
    }
    datRIndex = 0x00;
    for (i=0; i<20; i++)
    {
        datW[i] = i;
    }
    datWIndex = 0x00;

    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();
    
    /*I2C初始化（P31做SCL，P32做SDA）*/
    I2CInit();

    /*使能总中断*/
    EA = 1;
    
    while (1)
    {
        /*如果P07=0，则printf接收到的数据（注：主机P07和从机P07接在一起，主机读、写完成后会拉低P07）（仅用于测试）*/
        if (0 == P0_7)
        {
            datRIndex = 0x00;
            
            for (i=0; i<50; i++)
            {
                printf("%02X ", (int)datR[i]);
                datR[i] = 0x00;
            }
            printf("\r\n\r\n");
            
            while (0 == P0_7);
        }
    }
}

/*I2C中断处理程序*/
void I2CInt(void) interrupt 21
{
    uint8_t i2cssr = I2CSCR;
    
    uint8_t i = 0x00;
    
    /*TREQ=1，代表接收到来自主机的“读”请求*/
    if (i2cssr & 0x02)
    {
        I2CSBUF = datW[datWIndex ++];
        if (datWIndex >= 20)
        {
            datWIndex = 0x00;
        }
    }
    
    /*SENDFIN=1，代表接收到来自主机的“结束读”请求*/
    if (i2cssr & 0x04)
    {
        datWIndex = 0x00;
        
        printf("Master has stopped to read slave\r\n");
    }
    
    /*RREQ=1，代表接收到来自主机的数据，包含slaDatAddr和dat[]*/
    if (i2cssr & 0x01)
    {
        datR[datRIndex ++] = I2CSBUF;
        if (datRIndex >= 80)
        {
            datRIndex = 0x00;
        }
       
//         printf("%02X ", (int)datR[datRIndex]);
    }
}