#include "common.h"

/*注：实测休眠功耗52.2uA（仅代表该颗IC，不代表批量数据）*/

/*GPIO中断初始化（P43下降沿中断）*/
void GPIOIntInit(void)
{
    /*P43内部上拉输入*/
    P4TRIS &= ~BIT3;
    P4UP |= BIT3;
    P4_3 = 1;
    
    /*下降沿触发中断*/
    P43EICFG = 0x02;
    
    /*清标志*/
    P4EXTIF &= ~BIT3;
    
    /*使能中断*/
    P4EXTIE |= BIT3;
}

/*LCD所有SEG打开*/
void LCDAllSEGOn(void)
{
    LCDSEG0 = 0x0F;
    LCDSEG1 = 0x0F;
    LCDSEG2 = 0x0F;
    LCDSEG3 = 0x0F;
    LCDSEG4 = 0x0F;
    LCDSEG5 = 0x0F;
    LCDSEG6 = 0x0F;
    LCDSEG7 = 0x0F;
    LCDSEG8 = 0x0F;
    LCDSEG9 = 0x0F;
    LCDSEG10 = 0x0F;
    LCDSEG11 = 0x0F;
}

/*LCD初始化（COM0~COM3选择P30~P33；SEG0~SEG11选择P10~P17和P20~P23）*/
void LCDInit(void)
{
    /*使能LCD；正常输出；模式0；1/4 duty；COM顺序是C0~C3，SEG有效口是S0~S35*/
    LCDCON0 = BIT7 | BIT5 | BIT4 | BIT1 | BIT0;
    
    /*LCD电源电压由VDD提供；1/3 bias*/
    LCDCON1 = BIT4;
    
    /*LCD时钟来源=LSI/512*/
    LCDCON2 = (BIT5 | BIT1 | BIT0);
    
    /*LCD分压电阻=60K*/
    LCDCON3 = 0x00;
    
    /*使能COM0~COM3*/
    LCDCOMEN = BIT3 | BIT2 | BIT1 | BIT0;
    
    /*使能SEG0~SEG11*/
    LCDSEGEN0 = 0xFF;
    LCDSEGEN1 = BIT3 | BIT2 | BIT1 | BIT0;
    LCDSEGEN2 = 0x00;
    LCDSEGEN3 = 0x00;
    LCDSEGEN4 = 0x00;
    
    /*P30~P33做COM0~COM3；P10~P17和P20~P23做SEG0~SEG11*/
    P30CFG = 0x01;
    P31CFG = 0x01;
    P32CFG = 0x01;
    P33CFG = 0x01;
    
    P10CFG = 0x01;
    P11CFG = 0x01;
    P12CFG = 0x01;
    P13CFG = 0x01;
    P14CFG = 0x01;
    P15CFG = 0x01;
    P16CFG = 0x01;
    P17CFG = 0x01;
    P20CFG = 0x01;
    P21CFG = 0x01;
    P22CFG = 0x01;
    P23CFG = 0x01;
}

/*WUT初始化（本例的目的是为了休眠状态下LSI能驱动LCD）*/
void WUTInit(void)
{
    /*定时时间设置为最长*/
    WUTCRH = BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;
    WUTCRL = 0xFF;
    
    /*打开定时唤醒功能*/
    WUTCRH |= BIT7;
}

/*休眠*/
void Powerdown(void)
{
    /*悬空的IO要设置成输出；非悬空的IO要关闭耗电（注：本例除P10~P17、P20~P23、P30~P33、P43、P53外，其他IO均悬空）*/
    P0TRIS = 0xFF;
    P2TRIS |= 0xF0;
    P3TRIS |= 0xF0;
    P4TRIS |= 0xF7;
    P5TRIS |= 0xF7;
    
    P0OD = 0x00;
    P2OD &= 0x0F;
    P3OD &= 0x0F;
    P4OD &= 0x08;
    P5OD &= 0x08;
    
    /*休眠，且允许外部中断或定时唤醒*/
    PCON |= (BIT2 | BIT1);
    _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
}

/*main*/
void main(void)
{
    uint16_t i = 0x0000;
    
    /*P53推挽输出（仅用于测试）*/
    P5TRIS |= BIT3;
    P5OD &= ~BIT3;
    P5_3 = 0;
    
    /*GPIO中断初始化（P43下降沿中断）*/
    GPIOIntInit();
    
    /*LCD初始化（COM0~COM3选择P30~P33；SEG0~SEG11选择P10~P17和P20~P23）*/
    LCDInit();
    
    /*使能总中断*/
    EA = 1;
    
    /*LCD所有SEG打开*/
    LCDAllSEGOn();
    
    /*WUT初始化（本例的目的是为了休眠状态下LSI能驱动LCD）*/
    WUTInit();
    
    /*休眠*/
    Powerdown();
    
    while (1)
    {
        /*翻转P53（仅用于测试）*/
        P5_3 = ~P5_3;
        
        /*休眠*/
        Powerdown();
    }
}

/*P4中断处理程序*/
void P4IntInt(void) interrupt 11
{
    /*P43中断*/
    if (P4EXTIF & BIT3)
    {
        /*清标志*/
        P4EXTIF &= ~BIT3;
    }
}