#include "common.h"

/*PWM初始化（P03、P02、P01、P00、P05、P04分别做PWM0~PWM5，输出频率/占空比分别为10K/25%、20K/33.33%、12.5K/50%、3.75K/75%、10K/25%、20K/80%，当Fsys=48MHz时）*/
void PWMInit(void)
{
    /*P00~P05推挽输出*/
    P0TRIS |= (BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0);
    P0OD &= ~(BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0);
    P0 |= (BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0);
    
    /*P03、P02、P01、P00、P05、P04分别做PWM0~PWM5管脚*/
    P00CFG = 0x05;
    P01CFG = 0x05;
    P02CFG = 0x05;
    P03CFG = 0x05;
    P04CFG = 0x05;
    P05CFG = 0x05;
    
    /*独立模式、边沿对齐*/
    PWMCON = 0x00;
    
    /*PWM01、PWM23、PWM45时钟来源=Fsys/(PWMnmPSC+1)*/
    PWM01PSC = 3;
    PWM23PSC = 7;
    PWM45PSC = 11;
    
    /*PWM0~PWM5时钟来源*/
    PWM0DIV = 0;//PWM0时钟来源=PWM01时钟来源/2
//     PWM0DIV = 1;//PWM0时钟来源=PWM01时钟来源/4
//     PWM0DIV = 2;//PWM0时钟来源=PWM01时钟来源/8
//     PWM0DIV = 3;//PWM0时钟来源=PWM01时钟来源/16
//     PWM0DIV = 4;//PWM0时钟来源=PWM01时钟来源
//     PWM0DIV = 5;//PWM0时钟来源=Fsys
    
    PWM1DIV = 1;//PWM1~PWM5时钟来源用法同PWM0
    PWM2DIV = 2;
    PWM3DIV = 3;
    PWM4DIV = 4;
    PWM5DIV = 5;
    
    /*PWM0~PWM5周期=({PWMPnH,PWMPnH}+1)/PWMn时钟来源*/
    #define PWMP0H_PWMP0L 599//10KHz
    PWMP0 = (PWMP0H_PWMP0L >> 8) & 0x00FF;
    PWMP0L = PWMP0H_PWMP0L & 0x00FF;
    
    #define PWMP1H_PWMP1L 149//20KHz
    PWMP1 = (PWMP1H_PWMP1L >> 8) & 0x00FF;
    PWMP1L = PWMP1H_PWMP1L & 0x00FF;
    
    #define PWMP2H_PWMP2L 59//12.5KHz
    PWMP2 = (PWMP2H_PWMP2L >> 8) & 0x00FF;
    PWMP2L = PWMP2H_PWMP2L & 0x00FF;
    
    #define PWMP3H_PWMP3L 99//3.75KHz
    PWMP3 = (PWMP3H_PWMP3L >> 8) & 0x00FF;
    PWMP3L = PWMP3H_PWMP3L & 0x00FF;
    
    #define PWMP4H_PWMP4L 399//10KHz
    PWMP4 = (PWMP4H_PWMP4L >> 8) & 0x00FF;
    PWMP4L = PWMP4H_PWMP4L & 0x00FF;
    
    #define PWMP5H_PWMP5L 2399//20KHz
    PWMP5 = (PWMP5H_PWMP5L >> 8) & 0x00FF;
    PWMP5L = PWMP5H_PWMP5L & 0x00FF;
    
    /*PWM0~PWM5占空比*/
    #define PWMD0H_PWMD0L 149//25%
    PWMD0 = (PWMD0H_PWMD0L >> 8) & 0x00FF;
    PWMD0L = PWMD0H_PWMD0L & 0x00FF;
    
    #define PWMD1H_PWMD1L 49//33.33%
    PWMD1 = (PWMD1H_PWMD1L >> 8) & 0x00FF;
    PWMD1L = PWMD1H_PWMD1L & 0x00FF;
    
    #define PWMD2H_PWMD2L 29//50%
    PWMD2 = (PWMD2H_PWMD2L >> 8) & 0x00FF;
    PWMD2L = PWMD2H_PWMD2L & 0x00FF;
    
    #define PWMD3H_PWMD3L 74//75%
    PWMD3 = (PWMD3H_PWMD3L >> 8) & 0x00FF;
    PWMD3L = PWMD3H_PWMD3L & 0x00FF;
    
    #define PWMD4H_PWMD4L 99//25%
    PWMD4 = (PWMD4H_PWMD4L >> 8) & 0x00FF;
    PWMD4L = PWMD4H_PWMD4L & 0x00FF;
    
    #define PWMD5H_PWMD5L 1919//80%
    PWMD5 = (PWMD5H_PWMD5L >> 8) & 0x00FF;
    PWMD5L = PWMD5H_PWMD5L & 0x00FF;
 
    /*自动加载模式*/
    PWMCNTM = 0x3F;
    
    /*不反向*/
    PWMPINV = 0x00;
    
    /*使能PWM加载周期、占空比数据*/
    PWMLOADEN = 0x3F;
    
    /*使能PWM输出*/
    PWMOE = 0x3F;
    
    /*清零计数器*/
    PWMCNTCLR = 0x3F;
    
    /*启动计数器*/
    PWMCNTE = 0x3F;
}

/*main*/
void main(void)
{
    /*PWM初始化（P03、P02、P01、P00、P05、P04分别做PWM0~PWM5，输出频率/占空比分别为10K/25%、20K/33.33%、12.5K/50%、3.75K/75%、10K/25%、20K/80%，当Fsys=48MHz时）*/
    PWMInit();
    
    while (1);
}