#include "common.h"

/*SPI发送并接收*/
uint8_t SPISendAndReceive(uint8_t datS, uint8_t* pDatR)
{
    uint8_t spsr = 0x00;
    
    /*发送并接收，并等待直到传输完成或写冲突*/
    SPDR = datS;
    while (0x00 == (EIF2&BIT7));
    
    spsr = SPSR;
    *pDatR = SPDR;
    
    if (spsr & BIT6)//写冲突
    {
        *pDatR = 0x00;
        return 0;
    }

    return 1;
}

/*SPI初始化（主机模式；P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚；时钟来源=Fsys/16；SCLK空闲时为低电平；SCLK下降沿采数据上升沿放数据；NSS空闲时为高电平，传输启动时为低电平）*/
void SPIInit(void)
{
    uint8_t spsr = 0x00;
    uint8_t spdr = 0x00;
    
    /*使能SPI模块；主机模式；时钟来源=Fsys/8；SCLK空闲时为低电平（CPOL=0）；SCLK下降沿采数据上升沿放数据（CPHA=1）*/
    SPCR = BIT6 | BIT4 | BIT2 | BIT0;
    
    /*NSSO1空闲时为高电平，传输启动时为低电平*/
    SSCR = ~BIT1;
    SPSR |= BIT0;
    
    /*P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚*/
    P13CFG = 0x03;
    P14CFG = 0x03;
    P12CFG = 0x03;
    P15CFG = 0x03;
    
    PS_MOSI = 0x13;
    PS_MISO = 0x14;
    PS_SCLK = 0x12;
    PS_NSS = 0x15;
    
    /*P13、P12、P15推挽输出*/
    P1TRIS |= (BIT5 | BIT3 | BIT2);
    P1OD &= ~(BIT5 | BIT3 | BIT2);
    
    /*P14带内部上拉做输入*/
    P1TRIS &= ~BIT4;
    P1UP |= BIT4;
    
    /*清标志*/
    spsr = SPSR;
    spdr = SPDR;
}

/*main*/
void main(void)
{
    uint8_t datS = 0x00;
    uint8_t datR = 0x00;
    
    uint16_t i = 0x0000;
    
    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();
    
    /*SPI初始化（主机模式；P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚；时钟来源=Fsys/16；SCLK空闲时为低电平；SCLK下降沿采数据上升沿放数据；NSS空闲时为高电平，传输启动时为低电平）*/
    SPIInit();
    
    /*延时一段时间，等待SPI从机准备好*/
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
       
    while (1)
    {
        /*SPI发送并接收*/
        if (0 == SPISendAndReceive(datS,&datR))
        {
            printf("\r\nSPI error\r\n");
            while (1);
        }
        
        printf("%02X\r\n", (int)datR);
        
        datS ++;
        datR = 0x00;
        
        for (i=0; i<60000; i++);for (i=0; i<60000; i++);for (i=0; i<60000; i++);
    }
}