#include "common.h"

/*SPI初始化（从机模式；P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚；SCLK空闲时为低电平；SCLK下降沿采数据上升沿放数据；NSS空闲时为高电平，传输启动时为低电平）*/
void SPIInit(void)
{
    uint8_t spsr = 0x00;
    uint8_t spdr = 0x00;
    
    /*使能SPI模块；从机模式；SCLK空闲时为低电平（CPOL=0）；SCLK下降沿采数据上升沿放数据（CPHA=1）*/
    SPCR = BIT6 | BIT2;
    
    /*NSSO1空闲时为高电平，传输启动时为低电平*/
    SSCR = ~BIT1;
    SPSR |= BIT0;
    
    /*准备第一个要发送的数据（本例用0xA5）*/
    SPDR = 0xA5;
    
    /*P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚*/
    P13CFG = 0x03;
    P14CFG = 0x03;
    P12CFG = 0x03;
    P15CFG = 0x03;
    
    PS_MOSI = 0x13;
    PS_MISO = 0x14;
    PS_SCLK = 0x12;
    PS_NSS = 0x15;
    
    /*P13、P12、P15带内部上拉做输入*/
    P1TRIS &= ~(BIT5 | BIT3 | BIT2);
    P1UP |= (BIT5 | BIT3 | BIT2);
    
    /*P14推挽输出*/
    P1TRIS |= BIT4;
    P1OD &= ~BIT4;
    
    /*清标志*/
    spsr = SPSR;
    spdr = SPDR;
    
    /*使能中断*/
    EIE2 |= BIT7;
}

/*main*/
void main(void)
{
    /*UART0初始化（P21做TX，波特率115200源于BRT，当Fsys=48MHz时）*/
    UART0Init();
    
    /*SPI初始化（从机模式；P13、P14、P12、P15分别做MOSI、MISO、SCLK、NSSO1管脚；SCLK空闲时为低电平；SCLK下降沿采数据上升沿放数据；NSS空闲时为高电平，传输启动时为低电平）*/
    SPIInit();
    
    /*使能总中断*/
    EA = 1;
    
    while (1);
}

/*SPI中断处理程序*/
void SPIInt(void) interrupt 22
{
    static uint8_t datS = 0x00;
    uint8_t datR = 0x00;
    
    uint8_t spsr = 0x00;
    uint8_t spdr = 0x00;
    
    spsr = SPSR;
    spdr = SPDR;
    
    if (spsr & BIT6)//写冲突中断
    {
        printf("\r\nSPI error\r\n");
    }
    else//传输完成中断
    {        
        /*接收到的数据*/
        datR = spdr;
        
        printf("%02X\r\n", (int)datR);
        
        /*准备下一个要发送的数据*/
        SPDR = datS ++;
    }
}