#include "common.h"

#define TH0_RELOAD_VALUE 0xFF
#define TL0_RELOAD_VALUE 0x38

/*Timer0初始化（16bit手动重载，溢出频率60KHz，当Fsys=48MHz时）*/
void Timer0Init(void)
{
    /*8bit自动重载*/
    TMOD &= 0xF0;
    TMOD |= BIT0;
    
    /*时钟来源=Fsys/4*/
    CKCON |= BIT3;
    
    /*溢出频率=时钟来源/(2^16-{TH0,TL0})，本例=48MHz/4/(65536-65336)=60KHz*/
    TH0 = TH0_RELOAD_VALUE;
    TL0 = TL0_RELOAD_VALUE;
    
    /*清标志*/
    TF0 = 0;
    
    /*使能中断*/
    ET0 = 1;
}

/*main*/
void main(void)
{
    /*P04推挽输出（仅用于测试）*/
    P0TRIS |= BIT4;
    P0OD &= ~BIT4;
    
    /*Timer0初始化（16bit手动重载，溢出频率60KHz，当Fsys=48MHz时）*/
    Timer0Init();
    
    /*使能总中断*/
    EA = 1;
    
    /*启动定时器0*/
    TR0 = 1;
    
    while (1);
}

/*Timer0中断处理程序*/
void Timer0Int(void) interrupt 1
{    
    /*重载TH0、TL0*/
    TL0 = TL0_RELOAD_VALUE;
    TH0 = TH0_RELOAD_VALUE;
    
    /*清标志*/
    TF0 = 0;
    
    /*翻转P04（仅用于测试）*/
    P0_4 = ~P0_4;
}