#include "common.h"

/*UART0发送*/
void UART0Send(uint8_t dat)
{
    uint8_t sfrs = SFRS;
    SFRS = 0x00;
    
    SBUF0 = dat;
    while (0x00 == (SCON0&BIT1));
    SCON0 &= ~BIT1;
    
    SFRS = sfrs;
}

/*UART0初始化（P21做TX，P20做RX，波特率115200源于BRT，当Fsys=48MHz时）*/
void UART0Init(void)
{
    /*8bit异步模式，波特率可变*/
    SCON0 = BIT6;
    
    /*波特率来源于BRT*/
    FUNCCR &= ~BIT2;
    FUNCCR |= (BIT1 | BIT0);
    
    /*波特率115200，当Fsys=48MHz时*/
    PCON &= ~BIT7;
    BRTCON = BIT7;
    #define BRTDH_BRTDL 65523
    BRTD = (BRTDH_BRTDL >> 8) & 0x00FF;
    BRTDL = BRTDH_BRTDL & 0x00FF;
    
    /*P21做TX、P20做RX*/
    P21CFG = 0x02;
    P20CFG = 0x02;
    
    P2TRIS |= BIT1;
    P2OD &= ~BIT1;
    P2_1 = 1;
    
    P2TRIS &= ~BIT0;
    P2UP |= BIT0;
    P2_0 = 1;
    
    /*使能接收*/
    SCON0 |= BIT4;
    
    /*清标志*/
    SCON0 &= ~(BIT1 | BIT0);
    
    /*使能中断*/
    IE |= BIT4;
}

/*UART0中断处理程序*/
void UART0Int(void) interrupt 4
{
    uint8_t sfrs = SFRS;
    SFRS = 0x00;
    
    /*接收中断*/
    if (SCON0 & BIT0)
    {
        /*清标志*/
        SCON0 &= ~BIT0;
        
        /*通过UART0发送回去（仅用于测试）*/
        UART0Send(SBUF0);
    }
    
    SFRS = sfrs;
}