#include "common.h"

/*UART2发送*/
void UART2Send(uint8_t dat)
{
    uint8_t sfrs = SFRS;
    SFRS = 0x01;
    
    SBUF2 = dat;
    while (0x00 == (SCON2&BIT1));
    SCON2 &= ~BIT1;
    
    SFRS = sfrs;
}

/*UART2初始化（P05做TX，P06做RX，波特率115200，当Fsys=48MHz时）*/
void UART2Init(void)
{
    /*8bit异步模式，波特率可变*/
    SFRS = 0x01;
    SCON2 = BIT6;
    SFRS = 0x00;

    /*波特率来源于BRT*/
    SFRS = 0x01;
    FUNCCR1 &= ~BIT2;
    FUNCCR1 |= (BIT1 | BIT0);
    SFRS = 0x00;

    /*波特率115200，当Fsys=48MHz时*/
    SFRS = 0x01;
    PCON1 &= ~BIT0;
    SFRS = 0x00;
    BRTCON = BIT7;
    #define BRTDH_BRTDL 65523
    BRTD = (BRTDH_BRTDL >> 8) & 0x00FF;
    BRTDL = BRTDH_BRTDL & 0x00FF;

    /*P05做TX、P06做RX*/
    P05CFG = 0x02;
    P06CFG = 0x02;
    
    P0TRIS |= BIT5;
    P0OD &= ~BIT5;
    P0_5 = 1;
    
    P0TRIS &= ~BIT6;
    P0UP |= BIT6;
    P0_6 = 1;
    
    /*使能接收*/
    SFRS = 0x01;
    SCON2 |= BIT4;
    SFRS = 0x00;
    
    /*清标志*/
    SFRS = 0x01;
    SCON2 &= ~(BIT1 | BIT0);
    SFRS = 0x00;
    
    /*使能中断*/
    EIE3 |= BIT0;
}

/*main*/
void main(void)
{
    /*UART2初始化（P05做TX，P06做RX，波特率115200，当Fsys=48MHz时）*/
    UART2Init();
    
    /*使能总中断*/
    EA = 1;

    while (1);
}

/*UART2中断处理程序*/
void UART2Int(void) interrupt 23
{
    uint8_t sfrs = SFRS;
    SFRS = 0x01;
    
    /*接收中断*/
    if (SCON2 & BIT0)
    {
        /*清标志*/
        SCON2 &= ~BIT0;
        
        /*通过UART2发送回去（仅用于测试）*/
        UART2Send(SBUF2);
    }
    
    SFRS = sfrs;
}