#include "common.h"

/*UART3发送*/
void UART3Send(uint8_t dat)
{
    uint8_t sfrs = SFRS;
    SFRS = 0x01;
    
    SBUF3 = dat;
    while (0x00 == (SCON3&BIT1));
    SCON3 &= ~BIT1;
    
    SFRS = sfrs;
}

/*UART3初始化（P44做TX，P45做RX，波特率115200，当Fsys=48MHz时）*/
void UART3Init(void)
{
    /*8bit异步模式，波特率可变*/
    SFRS = 0x01;
    SCON3 = BIT6;
    SFRS = 0x00;

    /*波特率来源于BRT*/
    SFRS = 0x01;
    FUNCCR1 &= ~BIT6;
    FUNCCR1 |= (BIT5 | BIT4);
    SFRS = 0x00;

    /*波特率115200，当Fsys=48MHz时*/
    SFRS = 0x01;
    PCON1 &= ~BIT1;
    SFRS = 0x00;
    BRTCON = BIT7;
    #define BRTDH_BRTDL 65523
    BRTD = (BRTDH_BRTDL >> 8) & 0x00FF;
    BRTDL = BRTDH_BRTDL & 0x00FF;

    /*P44做TX、P45做RX*/
    P44CFG = 0x02;
    P45CFG = 0x02;
    
    P4TRIS |= BIT4;
    P4OD &= ~BIT4;
    P4_4 = 1;
    
    P4TRIS &= ~BIT5;
    P4UP |= BIT5;
    P4_5 = 1;
    
    /*使能接收*/
    SFRS = 0x01;
    SCON3 |= BIT4;
    SFRS = 0x00;
    
    /*清标志*/
    SFRS = 0x01;
    SCON3 &= ~(BIT1 | BIT0);
    SFRS = 0x00;
    
    /*使能中断*/
    EIE3 |= BIT1;
}

/*main*/
void main(void)
{
    /*UART3初始化（P44做TX，P45做RX，波特率115200，当Fsys=48MHz时）*/
    UART3Init();
    
    /*使能总中断*/
    EA = 1;

    while (1);
}

/*UART3中断处理程序*/
void UART3Int(void) interrupt 24
{
    uint8_t sfrs = SFRS;
    SFRS = 0x01;
    
    /*接收中断*/
    if (SCON3 & BIT0)
    {
        /*清标志*/
        SCON3 &= ~BIT0;
        
        /*通过UART3发送回去（仅用于测试）*/
        UART3Send(SBUF3);
    }
    
    SFRS = sfrs;
}