#include "common.h"

/*注：实测休眠功耗4.9uA（仅代表该颗IC，不代表批量数据）*/

/*GPIO中断初始化（P43下降沿中断）*/
void GPIOIntInit(void)
{
    /*P43内部上拉输入*/
    P4TRIS &= ~BIT3;
    P4UP |= BIT3;
    P4_3 = 1;
    
    /*下降沿触发中断*/
    P43EICFG = 0x02;
    
    /*清标志*/
    P4EXTIF &= ~BIT3;
    
    /*使能中断*/
    P4EXTIE |= BIT3;
}

/*休眠*/
void Powerdown(void)
{
    /*悬空的IO要设置成输出；非悬空的IO要关闭耗电（注：本例除P43、P53外，其他IO均悬空）*/
    P0TRIS = 0xFF;
    P1TRIS = 0xFF;
    P2TRIS = 0xFF;
    P3TRIS = 0xFF;
    P4TRIS |= 0xF7;
    P5TRIS |= 0xF7;
    
    P0OD = 0x00;
    P1OD = 0x00;
    P2OD = 0x00;
    P3OD = 0x00;
    P4OD &= 0x08;
    P5OD &= 0x08;
    
    /*休眠，且允许外部中断或定时唤醒*/
    PCON |= (BIT2 | BIT1);
    _nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
}

/*main*/
void main(void)
{
    uint16_t i = 0x0000;
    
    /*P53推挽输出（仅用于测试）*/
    P5TRIS |= BIT3;
    P5OD &= ~BIT3;
    P5_3 = 0;
    
    /*GPIO中断初始化（P43下降沿中断）*/
    GPIOIntInit();
    
    /*使能总中断*/
    EA = 1;
    
    /*休眠*/
    Powerdown();
    
    while (1)
    {
        /*翻转P53（仅用于测试）*/
        P5_3 = ~P5_3;
        for (i=0; i<40000; i++);
    }
}

/*P4中断处理程序*/
void P4IntInt(void) interrupt 11
{
    /*P43中断*/
    if (P4EXTIF & BIT3)
    {
        /*清标志*/
        P4EXTIF &= ~BIT3;
    }
}